/*
 * @Author: donghao donghao@supervision.ltd
 * @Date: 2024-04-08 16:57:30
 * @LastEditors: donghao donghao@supervision.ltd
 * @LastEditTime: 2024-05-10 15:05:25
 * @FilePath: \general-ai-manage\src\pages\Project\BusinessProject\components\detailServerState.tsx
 * @Description: 服务器详情
 * @交互说明
 *
 */
import { bsServerDetailEnums } from '@/enums/server';
import { getModelGroupBaseModelList } from '@/services/testApi/model';
import { isSuccessApi } from '@/utils/forApi';
import { ProCard, ProDescriptions, ProList, ProTable } from '@ant-design/pro-components';
import { FormattedMessage, useIntl } from '@umijs/max';
import { Modal, Tabs } from 'antd';
import * as echarts from 'echarts';
import ReactEcharts from 'echarts-for-react';
import { useEffect, useState } from 'react';
import { proFormMaxModelWidth } from '../../../../../config/defaultForm';
import DetailServerStateLog from './detailServerStateLog';
type DetailServerStateProps = {
  info: Record<string, any>;
  detailOpen: boolean;
  closeModal: () => void;
};

const DetailServerState: React.FC<DetailServerStateProps> = ({ info, detailOpen, closeModal }) => {
  /**state */
  const intl = useIntl();
  const [tabKey, setTabKey] = useState<string>(bsServerDetailEnums[0].key);
  const [tabs] = useState<DICTENUM.DICT_TAB_ITEM[]>([...bsServerDetailEnums]);
  const [modelData, setModelData] = useState<Record<string, any>[]>([]); // 列表数据
  const [forceRender, setForceRender] = useState<boolean>(false);

  // 基础模型信息
  const ModelDetailColumns = [
    {
      title: <FormattedMessage id="server_state.table.form.name" defaultMessage="服务器名称" />,
      dataIndex: 'deviceSort',
    },
    {
      title: <FormattedMessage id="server_state.table.form.ip" defaultMessage="IP" />,
      dataIndex: 'IP',
    },
    {
      title: (
        <FormattedMessage id="server_state.table.form.defaultPort" defaultMessage="默认端口" />
      ),
      dataIndex: 'port',
    },
    {
      title: <FormattedMessage id="server_state.table.form.userName" defaultMessage="用户名" />,
      dataIndex: 'userName',
    },
    {
      title: <FormattedMessage id="server_state.table.form.pwd" defaultMessage="密码" />,
      dataIndex: 'pwd',
      render: () => {
        return '*********';
      },
    },
  ];

  // 硬件信息环形图
  const fetchOptionByData = (record) => {
    const totalCount = 100 * Math.floor(Math.random() * 15);
    const currData = [
      {
        value: record?.percent * totalCount,
        name: `${intl.formatMessage({
          id: 'server_state.table.detail.echarts.used',
          defaultMessage: '已占用',
        })}`,
      },
      {
        value: totalCount - record?.percent * totalCount,
        name: `${intl.formatMessage({
          id: 'server_state.table.detail.echarts.free',
          defaultMessage: '未占用',
        })}`,
      },
    ];

    let currNames = [];
    currData.forEach((item) => {
      currNames.push(item.name);
    });
    console.log(record, 'fetchOptionByData_record');
    return {
      title: {
        text: record.label,
        x: 'center',
        y: '16.2%',
        textStyle: {
          fontSize: 16, // 标题文字大小
        },
      },
      tooltip: {
        trigger: 'item',
        formatter: '{a} <br/>{b} : {c} ({d}%)',
      },
      legend: {
        orient: 'horizontal',
        top: '37%',
        data: currNames,
      },
      series: [
        {
          name: '名称',
          type: 'pie',
          center: ['50%', '20%'], // 调整 center 属性使饼图偏上
          radius: ['28%', '45%'], // 控制环形图大小
          color: [
            new echarts.graphic.LinearGradient(0, 0, 0, 1, [
              {
                offset: 0,
                color: record.usedColors[0],
              },
              {
                offset: 1,
                color: record.usedColors[1],
              },
            ]),
            new echarts.graphic.LinearGradient(0, 0, 0, 1, [
              {
                offset: 0,
                color: record.freeColors[0],
              },
              {
                offset: 1,
                color: record.freeColors[1],
              },
            ]),
          ],
          avoidLabelOverlap: false,
          label: {
            show: false,
          },
          emphasis: {
            label: {
              show: false,
              fontSize: '20',
              fontWeight: 'bold',
            },
          },
          labelLine: {
            show: false,
          },
          data: currData,
        },
      ],
    };
  };

  // 模型列表信息
  const columns: ProColumns<Record<string, any>>[] = [
    {
      title: <FormattedMessage id="base_model.table.list.name" defaultMessage="基础模型名称" />,
      dataIndex: 'name',
      hideInSearch: true,
      key: 'fixedName',
      fixed: 'left',
      width: '45%',
    },
    {
      title: <FormattedMessage id="base_model.table.list.version" defaultMessage="版本" />,
      dataIndex: 'version',
      hideInSearch: true,
      width: '20%',
    },
    {
      title: (
        <FormattedMessage id="base_model.table.list.runtimeLibFile" defaultMessage="运行库镜像" />
      ),
      dataIndex: 'runtimeLibFile',
      hideInSearch: true,
      render: (dom, record) => {
        return (
          <div>
            {record.runtimeLibFile ? (
              dom
            ) : (
              <div className={`gn_list_type_tag flex items-center justify-center active2`}>
                <span className="dot"></span>
                <span>
                  <FormattedMessage
                    id="base_model.table.list.undeployed.runtimeLibFile"
                    defaultMessage="未部署"
                  />
                </span>
              </div>
            )}
          </div>
        );
      },
    },
  ];

  // 将数据组装成reactDom
  function toListDomByData(record) {
    let startList = [...record];
    let finalList = startList.map((item) => ({
      content: (
        <div>
          <div className="gn_list_card_title pb-[12px]">
            <span></span>
            <span>{item.name}</span>
          </div>
          <ProTable
            className="gn_pro_table mb-[16px]"
            cardProps={{
              bodyStyle: { padding: 0, margin: 0 },
            }}
            // 标题栏
            search={false}
            options={{ fullScreen: false, setting: false, density: false, reload: false }}
            rowKey="id"
            onDataSourceChange={(data) => {
              console.log(data, 'onDataSourceChange_data');
            }}
            pagination={false}
            dataSource={item.list}
            columns={columns}
          />
        </div>
      ),
    }));
    setModelData(finalList);
    console.log(finalList, 'toListDomByData_finalList');
  }

  // 基础模型列表数据api
  async function fetchData() {
    const resp = await getModelGroupBaseModelList();
    if (isSuccessApi(resp)) {
      toListDomByData(resp.data.results);
    }
  }

  useEffect(() => {
    setForceRender(detailOpen);
  }, [detailOpen]);

  // 初始化加载
  useEffect(() => {
    fetchData();
  }, []);

  return (
    <Modal
      width={proFormMaxModelWidth}
      title={info?.deviceSort}
      open={detailOpen}
      onCancel={closeModal}
      footer={null}
      afterVisibleChange={(isVisible) => {
        if (!isVisible) {
          setForceRender(false); // Modal 关闭时重置状态
        }
      }}
    >
      <ProCard className="gn_card" bodyStyle={{ padding: 0 }}>
        <Tabs
          className="gn_tabs"
          activeKey={tabKey}
          items={tabs}
          onChange={(key) => {
            setTabKey(key);
          }}
        ></Tabs>
        {tabKey === '0' && (
          <div>
            <ProCard className="gn_card bs_server_info_wrap" bodyStyle={{ padding: 0 }}>
              <ul>
                <li className="pb-[8px]">
                  <p className="head4">
                    <FormattedMessage
                      id="server_state.table.detail.title.params"
                      defaultMessage="服务器参数"
                    />
                  </p>
                  <div className="my-[8px] gn_active_descriptions gn_descriptions bg_active_4">
                    <ProDescriptions
                      column={5}
                      columns={ModelDetailColumns}
                      dataSource={info}
                    ></ProDescriptions>
                  </div>
                </li>
                <li className="pb-[16px]">
                  <p className="head4">
                    <FormattedMessage
                      id="server_state.table.detail.title.info"
                      defaultMessage="硬件信息"
                    />
                  </p>
                  <ul className="mt-[8px] flex items-center ">
                    {detailOpen &&
                      info?.progressData?.map((item, index) => {
                        return (
                          <li
                            key={index}
                            className={`w-[25%] bg_active_4 rounded ${
                              index !== 0 ? 'ml-[10px]' : ''
                            }`}
                          >
                            <div style={{ height: '140px' }}>
                              {forceRender && (
                                <ReactEcharts echarts={echarts} option={fetchOptionByData(item)} />
                              )}
                            </div>
                          </li>
                        );
                      })}
                  </ul>
                </li>
                <li className="pb-[8px] bs_server_task_wrap">
                  <p className="head4">
                    <FormattedMessage
                      id="server_state.table.detail.title.task"
                      defaultMessage="当前任务"
                    />
                  </p>
                  <div className="pt-[8px]">
                    <ProList<{ title: string }>
                      className="gn_pro_list"
                      itemLayout="vertical"
                      itemCardProps={{
                        ghost: true,
                        bodyStyle: { padding: 0, margin: 0 },
                        style: {
                          width: '100%',
                          border: 0,
                        },
                      }}
                      cardProps={{
                        style: { padding: 0, margin: 0 }, // 设置卡片的内外边距为 0
                        bodyStyle: {
                          padding: 0,
                          margin: 0,
                          height: 'calc(300px)',
                          overflow: 'scroll',
                        },
                      }}
                      rowKey="id"
                      dataSource={modelData}
                      pagination={false}
                      rowSelection={false}
                      metas={{
                        content: {
                          style: { margin: 0, padding: 0 },
                        },
                      }}
                    />
                  </div>
                </li>
              </ul>
            </ProCard>
          </div>
        )}
        {tabKey === '1' && <DetailServerStateLog />}
      </ProCard>
    </Modal>
  );
};

export default DetailServerState;
